@@ -295,7 +295,9 @@ class OverlayNode(object):
 
     def add_loopback(self, *args, **kwargs):
         """Public function to add a loopback interface"""
-        self._add_interface(type="loopback", *args, **kwargs)
+        interface_id = self._add_interface(type="loopback", *args, **kwargs)
+        return overlay_interface(self.anm, self.overlay_id, 
+                self.node_id, interface_id)
 
     def add_interface(self,*args,  **kwargs):
         """Public function to add interface"""
@@ -627,11 +629,17 @@ class OverlayGraphData(object):
         return self.anm.overlay_nx_graphs[self.overlay_id]
 
     def __getattr__(self, key):
-        """Returns edge property"""
         return self._graph.graph.get(key)
 
     def __setattr__(self, key, val):
-        """Sets edge property"""
+        self._graph.graph[key] = val
+
+    def __getitem__(self, key):
+        """"""
+        return self._graph.graph.get(key)
+
+    def __setitem__(self, key, val):
+        """"""
         self._graph.graph[key] = val
 
 
