@@ -10,19 +10,12 @@ import autonetkit.plugins.naming as naming
 import autonetkit.config
 settings = autonetkit.config.settings
 from autonetkit.ank_utils import alphabetical_sort as alpha_sort
-
-
-def address_prefixlen_to_network(address, prefixlen):
-    """Workaround for creating an IPNetwork from an address and a prefixlen
-    TODO: check if this is part of netaddr module
-    """
-    return netaddr.IPNetwork("%s/%s" % (address, prefixlen))
+from autonetkit.ank import sn_preflen_to_network
 
 def dot_to_underscore(instring):
     """Replace dots with underscores"""
     return instring.replace(".", "_")
 
-
 class RouterCompiler(object):
     """Base router compiler"""
     lo_interface = "lo0"
@@ -133,14 +126,14 @@ class RouterCompiler(object):
                 ipv4_int = phy_int['ipv4']
                 interface.ipv4_address = ipv4_int.ip_address
                 interface.ipv4_subnet = ipv4_int.subnet
-                interface.ipv4_cidr = address_prefixlen_to_network(interface.ipv4_address,
+                interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
                         interface.ipv4_subnet.prefixlen)
 
             if node.ip.use_ipv6:
                 ipv6_int = phy_int['ipv6']
 #TODO: for consistency, make ipv6_cidr
                 interface.ipv6_subnet = ipv6_int.subnet
-                interface.ipv6_address = address_prefixlen_to_network(ipv6_int.ip_address,
+                interface.ipv6_address = sn_preflen_to_network(ipv6_int.ip_address,
                         interface.ipv6_subnet.prefixlen)
 
         for interface in node.loopback_interfaces:
@@ -154,14 +147,14 @@ class RouterCompiler(object):
                     ipv4_int = phy_int['ipv4']
                     interface.ipv4_address = ipv4_int.loopback
                     interface.ipv4_subnet = node.loopback_subnet
-                    interface.ipv4_cidr = address_prefixlen_to_network(interface.ipv4_address,
+                    interface.ipv4_cidr = sn_preflen_to_network(interface.ipv4_address,
                             interface.ipv4_subnet.prefixlen)
 
                 if node.ip.use_ipv6:
                     ipv6_int = phy_int['ipv6']
 #TODO: for consistency, make ipv6_cidr
                     #interface.ipv6_subnet = ipv6_int.loopback # TODO: do we need for consistency?
-                    interface.ipv6_address = address_prefixlen_to_network(
+                    interface.ipv6_address = sn_preflen_to_network(
                             ipv6_int.loopback, 128)
 
                 # secondary loopbacks
@@ -230,7 +223,6 @@ class RouterCompiler(object):
         ibgp_rr_clients = []
         ibgp_rr_parents = []
 
-
         g_ibgp_v4 = self.anm['ibgp_v4']
         for session in g_ibgp_v4.edges(phy_node):
             if session.exclude:
@@ -407,6 +399,8 @@ class IosBaseCompiler(RouterCompiler):
         self.vrf_igp_interfaces(node)
         phy_node = self.anm['phy'].node(node)
 
+        node.use_cdp = phy_node.use_cdp
+
         if node in self.anm['ospf']:
             node.ospf.use_ipv4 = phy_node.use_ipv4
             node.ospf.use_ipv6 = phy_node.use_ipv6
@@ -434,20 +428,24 @@ class IosBaseCompiler(RouterCompiler):
             ipv4_address = ipv4_loopback_zero.ip_address
             node.loopback_zero.ipv4_address = ipv4_address
             node.loopback_zero.ipv4_subnet = ipv4_loopback_subnet
-            node.loopback_zero.ipv4_cidr = address_prefixlen_to_network(
+            node.loopback_zero.ipv4_cidr = sn_preflen_to_network(
                     ipv4_address, ipv4_loopback_subnet.prefixlen)
 
         if node.ip.use_ipv6:
             ipv6_loopback_zero = phy_loopback_zero['ipv6']
-            node.loopback_zero.ipv6_address = address_prefixlen_to_network(
+            node.loopback_zero.ipv6_address = sn_preflen_to_network(
                 ipv6_loopback_zero.ip_address, 128)
 
         super(IosBaseCompiler, self).interfaces(node)
 
+        for interface in node.physical_interfaces:
+            interface.use_cdp = node.use_cdp # use node value
+
     def bgp(self, node):
         node.bgp.lo_interface = self.lo_interface
         super(IosBaseCompiler, self).bgp(node)
 
+        # Only advertise loopbacks into eBGP
         if node.ip.use_ipv4:
             node.bgp.ipv4_advertise_subnets = [node.loopback_zero.ipv4_cidr]
         if node.ip.use_ipv6:
@@ -527,9 +525,12 @@ class IosBaseCompiler(RouterCompiler):
             #TODO: check if mpls ldp already set elsewhere
             for vrf in vrf_node.node_vrf_names:
                 route_target = g_vrf.data.route_targets[node.asn][vrf]
+                rd_index = vrf_node.rd_indices[vrf]
+                rd = "%s:%s" % (node.asn, rd_index)
 
                 node.vrf.vrfs.append({
                     'vrf': vrf,
+                    "rd": rd,
                     'route_target': route_target,
                 })
 
@@ -547,6 +548,7 @@ class IosBaseCompiler(RouterCompiler):
                 mpls_ldp_int = self.anm['mpls_ldp'].interface(interface)
                 if mpls_ldp_int.is_bound:
                     node.mpls.ldp_interfaces.append(interface.id)
+                    interface.use_mpls = True
 
         if vrf_node.vrf_role is "P":
             node.mpls.ldp_interfaces = []
@@ -559,6 +561,10 @@ class IosBaseCompiler(RouterCompiler):
         node.vrf.use_ipv6 = node.ip.use_ipv6
         node.vrf.vrfs.sort("vrf")
 
+        if node in self.anm['mpls_ldp']:
+            node.mpls.enabled = True
+            node.mpls.router_id = node.loopback_zero.id
+
     def ospf(self, node):
         super(IosBaseCompiler, self).ospf(node)
         for interface in node.physical_interfaces:
@@ -586,7 +592,36 @@ class IosClassicCompiler(IosBaseCompiler):
         if phy_node.include_csr:
             node.include_csr = True
 
+    def bgp(self, node):
+        super(IosClassicCompiler, self).bgp(node)
+
+        vpnv4_neighbors = []
+        if node.bgp.vpnv4:
+            for neigh in node.bgp.ibgp_neighbors:
+                if not neigh.use_ipv4:
+                    continue
+                #TODO: fix up limitation where can't add as overlay_data
+                # (this causes problems serializing, when adding convert to dict?)
+                neigh_data = dict(neigh)
+                vpnv4_neighbors.append(neigh_data)
+
+            for neigh in node.bgp.ibgp_rr_clients:
+                if not neigh.use_ipv4:
+                    continue
+                neigh_data = dict(neigh)
+                neigh_data['rr_client'] = True
+                vpnv4_neighbors.append(neigh_data)
+
+            for neigh in node.bgp.ibgp_rr_parents:
+                if not neigh.use_ipv4:
+                    continue
+                neigh_data = dict(neigh)
+                vpnv4_neighbors.append(neigh_data)
+
+        node.bgp.vpnv4_neighbors = vpnv4_neighbors
+
 class Ios2Compiler(IosBaseCompiler):
+
     def ospf(self, node):
         super(Ios2Compiler, self).ospf(node)
         g_ospf = self.anm['ospf']
@@ -845,6 +880,11 @@ class CiscoCompiler(PlatformCompiler):
     def compile(self):
 #TODO: need to copy across the interface name from edge to the interface
         g_phy = self.anm['phy']
+        use_mgmt_interfaces = g_phy.data.mgmt_interfaces_enabled 
+        if use_mgmt_interfaces:
+            log.info("Allocating management interfaces for Cisco")
+        else:
+            log.info("Not allocating management interfaces for Cisco")
 
         log.info("Compiling Cisco for %s" % self.host)
         ios_compiler = IosClassicCompiler(self.nidb, self.anm)
@@ -857,10 +897,15 @@ class CiscoCompiler(PlatformCompiler):
 # TODO: merge common router code, so end up with three loops: routers, ios
 # routers, ios2 routers
 
+        # store autonetkit_cisco version
+        from pkg_resources import get_distribution
+        ank_cisco_version = get_distribution("autonetkit_cisco").version
+
         for phy_node in g_phy.nodes('is_router', host=self.host):
             loopback_ids = self.loopback_interface_ids()
             # allocate loopbacks to routes (same for all ios variants)
             nidb_node = self.nidb.node(phy_node)
+            nidb_node.ank_cisco_version = ank_cisco_version
 
             for interface in nidb_node.loopback_interfaces:
                 if interface != nidb_node.loopback_zero:
@@ -883,10 +928,13 @@ class CiscoCompiler(PlatformCompiler):
             elif phy_node.device_subtype == "ra":
                 int_ids = self.interface_ids_ra()
             else:
+                # default if no subtype specified
+                #TODO: need to set default in the load module
                 log.warning("Unexpected subtype %s" % phy_node.device_subtype)
                 int_ids = self.interface_ids_ios()
                 
-            mgmt_int_id = int_ids.next()  # 0/0 is used for management ethernet
+            if use_mgmt_interfaces:
+                mgmt_int_id = int_ids.next()  # 0/0 is used for management ethernet
 
             for interface in nidb_node.physical_interfaces:
                 if specified_int_names:
@@ -896,8 +944,9 @@ class CiscoCompiler(PlatformCompiler):
                     interface.id = int_ids.next()
 
             ios_compiler.compile(nidb_node)
-            mgmt_int = nidb_node.add_interface(management = True)
-            mgmt_int.id = mgmt_int_id
+            if use_mgmt_interfaces:
+                mgmt_int = nidb_node.add_interface(management = True)
+                mgmt_int.id = mgmt_int_id
 
         ios2_compiler = Ios2Compiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='ios2'):
@@ -922,9 +971,10 @@ class CiscoCompiler(PlatformCompiler):
 
             ios2_compiler.compile(nidb_node)
 
-            mgmt_int_id = "mgmteth 0/0/CPU0/0"
-            mgmt_int = nidb_node.add_interface(management = True)
-            mgmt_int.id = mgmt_int_id
+            if use_mgmt_interfaces:
+                mgmt_int_id = "mgmteth0/0/CPU0/0"
+                mgmt_int = nidb_node.add_interface(management = True)
+                mgmt_int.id = mgmt_int_id
 
         nxos_compiler = NxOsCompiler(self.nidb, self.anm)
         for phy_node in g_phy.nodes('is_router', host=self.host, syntax='nx_os'):
@@ -946,42 +996,17 @@ class CiscoCompiler(PlatformCompiler):
                 else:
                     interface.id = int_ids.next()
 
-            mgmt_int_id = "mgmt0"
-            mgmt_int = nidb_node.add_interface(management = True)
-            mgmt_int.id = mgmt_int_id
-            nxos_compiler.compile(nidb_node)
+            if use_mgmt_interfaces:
+                mgmt_int_id = "mgmt0"
+                mgmt_int = nidb_node.add_interface(management = True)
+                mgmt_int.id = mgmt_int_id
 
-        # assign management IPs
-        #TODO: make this a module
-        lab_topology = self.nidb.topology[self.host]
-        oob_management_ips = {}
-        from netaddr import IPNetwork
-        import netaddr
-        try:
-            management_subnet = address_prefixlen_to_network(g_phy.data.management_subnet,
-                    g_phy.data.management_prefixlen)
-        except (ValueError, netaddr.core.AddrFormatError):
-            management_subnet = IPNetwork("172.16.254.0/16")
-            log.info("Unable to create management subnet: %s/%s, using default of %s" % 
-                    (g_phy.data.management_subnet,
-                    g_phy.data.management_prefixlen, management_subnet))
-        management_ips = management_subnet.iter_hosts()
-        oob_host_ip = management_ips.next() # consume first ip for the host
-        oob_management_ips["host"] = oob_host_ip
-        for nidb_node in sorted(self.nidb.nodes('is_router', host=self.host)):
-            for interface in nidb_node.physical_interfaces:
-                if interface.management:
-                    interface.description = "OOB Management"
-                    interface.ipv4_address = management_ips.next()
-                    interface.ipv4_subnet = management_subnet
-                    interface.ipv4_cidr = address_prefixlen_to_network(interface.ipv4_address, management_subnet.prefixlen)
-                    interface.physical = True
-                    oob_management_ips[str(nidb_node)] = interface.ipv4_address
-        lab_topology.oob_management_ips = oob_management_ips
+            nxos_compiler.compile(nidb_node)
 
         other_nodes = [phy_node for phy_node in g_phy.nodes('is_router', host=self.host)
                        if phy_node.syntax not in ("ios", "ios2")]
         for node in other_nodes:
+            #TODO: check why we need this
             phy_node = g_phy.node(node)
             nidb_node = self.nidb.node(phy_node)
             nidb_node.input_label = phy_node.id  # set specifically for now for other variants
@@ -989,6 +1014,64 @@ class CiscoCompiler(PlatformCompiler):
 # TODO: use more os.path.join for render folders
 # TODO: Split compilers into seperate modules
 
+        if use_mgmt_interfaces:
+            self.assign_management_interfaces()
+
+    def assign_management_interfaces(self):
+        g_phy = self.anm['phy']
+        lab_topology = self.nidb.topology[self.host]
+        oob_management_ips = {}
+
+        #TODO: make this seperate function
+        from netaddr import IPNetwork, IPRange
+
+        mgmt_address_start = g_phy.data.mgmt_address_start 
+        mgmt_address_end = g_phy.data.mgmt_address_end 
+        mgmt_prefixlen = int(g_phy.data.mgmt_prefixlen)
+
+        #TODO: need to check if range is insufficient
+        mgmt_ips = (IPRange(mgmt_address_start, mgmt_address_end))
+        mgmt_ips_iter = iter(mgmt_ips) # to iterate over
+
+        mgmt_address_start_network = IPNetwork(mgmt_address_start) # as /32 for supernet
+        mgmt_address_end_network = IPNetwork(mgmt_address_end) # as /32 for supernet
+        # retrieve the first supernet, as this is the range requested. subsequent are the subnets
+        start_subnet = mgmt_address_start_network.supernet(mgmt_prefixlen)[0] # retrieve first
+        end_subnet = mgmt_address_end_network.supernet(mgmt_prefixlen)[0] # retrieve first
+
+        try: # validation
+            assert(start_subnet == end_subnet)
+            log.debug("Verified: Cisco management subnets match")
+        except AssertionError:
+            log.warning("Error: Cisco management subnets do not match: %s and %s, using start subnet"
+                    % (start_subnet, end_subnet))
+
+        mgmt_subnet = start_subnet
+        hosts_to_allocate = sorted(self.nidb.nodes('is_router', host=self.host))
+
+        try: # validation
+            assert(len(mgmt_ips) >= len(hosts_to_allocate))
+            log.debug("Verified: Cisco management IP range is sufficient size %s for %s hosts"
+                    % (len(mgmt_ips), len(hosts_to_allocate)))
+        except AssertionError:
+            log.warning("Error: Cisco management IP range is insufficient size %s for %s hosts"
+                    % (len(mgmt_ips), len(hosts_to_allocate)))
+            # TODO: need to use default range
+            return
+
+        for nidb_node in hosts_to_allocate:
+            for interface in nidb_node.physical_interfaces:
+                if interface.management:
+                    interface.description = "OOB Management"
+                    ipv4_address = mgmt_ips_iter.next()
+                    interface.ipv4_address = ipv4_address
+                    interface.ipv4_subnet = mgmt_subnet
+                    interface.ipv4_cidr = sn_preflen_to_network(ipv4_address, mgmt_prefixlen)
+                    interface.physical = True
+                    oob_management_ips[str(nidb_node)] = ipv4_address
+
+        lab_topology.oob_management_ips = oob_management_ips
+
 
 class DynagenCompiler(PlatformCompiler):
     """Dynagen Platform Compiler"""
