@@ -15,9 +15,6 @@ import autonetkit.ank_messaging as ank_messaging
 import autonetkit.config as config
 import autonetkit.ank_json as ank_json
 
-# import autonetkit.bgp_pol as bgp_pol
-# raise SystemExit
-
 # TODO: make if measure set, then not compile - or warn if both set, as
 # don't want to regen topology when measuring
 
@@ -26,7 +23,6 @@ try:
 except pkg_resources.DistributionNotFound:
     ANK_VERSION = "dev"
 
-
 def file_monitor(filename):
     """Generator based function to check if a file has changed"""
     last_timestamp = os.stat(filename).st_mtime
@@ -81,7 +77,7 @@ def manage_network(input_graph_string, timestamp, build_options, reload_build=Fa
         nidb = NIDB()
         nidb.restore_latest()
         update_http(anm, nidb)
-        
+
 
     if build_options['diff']:
         import autonetkit.diff
@@ -143,7 +139,6 @@ def parse_options():
     arguments = parser.parse_args()
     return arguments
 
-
 def main():
     settings = config.settings
 
@@ -170,7 +165,6 @@ def main():
         'archive': options.archive or settings['General']['archive'],
     }
 
-
     if options.webserver:
         log.info("Webserver not yet supported, please run as seperate module")
 
@@ -290,7 +284,7 @@ def compile_network(anm):
 def deploy_network(anm, nidb, input_graph_string = None):
 
     # TODO: make this driven from config file
-    log.info("Deploying network")
+    log.info("Deploying Network")
 
 # TODO: pick up platform, host, filenames from nidb (as set in there)
     deploy_hosts = config.settings['Deploy Hosts']
@@ -384,7 +378,7 @@ def measure_network(anm, nidb):
     # abort after 10 fails, proceed on any success, 0.1 second timeout (quite aggressive)
     if 1:
         collect_sh_ip_route(anm, nidb)
-        
+
 
     if 0:
         #measure.send(nidb, command, remote_hosts, threads = 5)
