@@ -14,7 +14,6 @@ from collections import defaultdict
 def load_graphml(input_data):
     #TODO: allow default properties to be passed in as dicts
 
-
     try:
         graph = nx.read_graphml(input_data)
     except IOError, e:
@@ -35,7 +34,15 @@ def load_graphml(input_data):
         else:
             raise e
 
-    # remove selfloops
+    if graph.is_multigraph():
+        log.info("Input graph is multigraph. Converting to single-edge graph")
+        if graph.is_directed():
+            graph = nx.DiGraph(graph)
+        else:
+            graph = nx.Graph(graph)
+
+    #TODO: need to support edge index keying for multi graphs
+
     graph.remove_edges_from(edge for edge in graph.selfloop_edges())
 #TODO: if selfloops then log that are removing
 
@@ -99,6 +106,16 @@ def load_graphml(input_data):
             if attr not in graph.node[node]:
                 graph.node[node][attr] = False
 
+    boolean_attributes = set( k for n1,d1 in graph.edge.items()
+                              for n2,d2 in d1.items()
+                              for k, v in d2.items()
+                              if isinstance(v, bool)
+    )
+    for n1,d1 in graph.edge.items():
+        for n2,d2 in d1.items():
+            for attr in boolean_attributes:
+                if attr not in graph.edge[n1][n2]:
+                    graph.edge[n1][n2][attr] = False
 
 #TODO: store these in config file
     ank_node_defaults = settings['Graphml']['Node Defaults']
@@ -153,6 +170,9 @@ def load_graphml(input_data):
     # and ensure asn is integer, x and y are floats
     for node in sorted(graph):
         graph.node[node]['asn'] = int(graph.node[node]['asn'])
+        if graph.node[node]['asn'] == 0:
+            log.debug("Node %s has ASN set to 0. Setting to 1" % graph.node[node]['label'])
+            graph.node[node]['asn'] = 1
         try:
             x = float(graph.node[node]['x'])
         except KeyError:
