@@ -198,6 +198,11 @@ class overlay_interface(object):
         """"""
         return self.type == "loopback" or self.phy.type == "loopback"
 
+    @property
+    def is_physical(self):
+        """"""
+        return self.type == "physical" or self.phy.type == "physical"
+
     @property
     def description(self):
         """"""
@@ -216,6 +221,11 @@ class overlay_interface(object):
     def dump(self):
         return str(self._interface.items())
 
+    def dict(self):
+        """Returns shallow copy of dictionary used.
+        Note not a deep copy: modifying values may have impact"""
+        return dict(self._interface.items())
+
     def __getattr__(self, key):
         """Returns interface property"""
         try:
@@ -437,7 +447,10 @@ class nidb_node_category(object):
         except AttributeError:
             pass # not a dict
         except TypeError:
-            log.debug("Access unset key %s in %s" % (key, self.node_id))
+            #Supress warning as could occur if this element is a primitive rather than a dict
+            #eg a boolean, which is valid
+            # TODO: revisit this
+            #log.debug("Access unset key %s in %s %s" % (key, self.node_id, self.category_id))
             pass # also not a dict
         return data
 
@@ -483,7 +496,13 @@ class nidb_node(object):
 
     def interface(self, key):
         #TODO: also need to allow access interface for nidb and search on (node, interface id) tuple
-        return overlay_interface(self.nidb, self.node_id, key.interface_id)
+        try:
+            interface_id = key.interface_id # eg extract from interface
+        except AttributeError:
+            interface_id = key # eg string
+
+        return overlay_interface(self.nidb, self.node_id, interface_id)
+
 
     @property
     def _interfaces(self):
@@ -494,13 +513,41 @@ class nidb_node(object):
             log.debug("No interfaces initialised for %s" % self)
             return 
 
+    @property
+    def _next_int_id(self):
+        """"""
+# returns next free interface ID
+        import itertools
+        for int_id in itertools.count(1):  # start at 1 as 0 is loopback
+            if int_id not in self._interfaces:
+                return int_id
+
+    def add_interface(self, description = None, type = "physical", *args,  **kwargs):
+        """Public function to add interface"""
+        data = dict(kwargs)
+        interface_id = self._next_int_id
+        data['type'] = type  # store type on node
+        data['description'] = description
+        self._interfaces[interface_id] = data
+
+        return overlay_interface(self.nidb, self.node_id, interface_id)
+
     @property
     def _interface_ids(self):
         return self._graph.node[self.node_id]["_interfaces"].keys()
 
     @property
     def interfaces(self):
-        return self.get_interfaces()
+        """Called by templates, sorts by ID"""
+        int_list = self.get_interfaces()
+
+        # Put loopbacks before physical interfaces
+        type_index = {"loopback": 0, "physical": 1}
+        #TODO: extend this based on medium type, etc
+
+        int_list = sorted(int_list, key = lambda x: x.id)
+        int_list = sorted(int_list, key = lambda x: type_index[x.type])
+        return int_list
 
     @property
     def physical_interfaces(self):
@@ -762,6 +809,11 @@ class NIDB_base(object):
                 json_fh.write(data)
 
 
+    def interface(self, interface):
+        return overlay_interface(self,
+                interface.node_id, interface.interface_id)
+
+
     def restore_latest(self, directory = None):
         import os
         import glob
@@ -841,6 +893,10 @@ class NIDB_base(object):
             for node in self:
                 if str(node) == key:
                     return node
+                elif node.id == key:
+                    # label could be "a b" -> "a_b" (ie folder safe, etc)
+                    #TODO: need to fix this discrepancy
+                    return node
             print "Unable to find node", key, "in", self
             return None
 
@@ -913,8 +969,12 @@ class NIDB_base(object):
 
         for node in nodes_to_add:
             #TODO: add an interface_retain for attributes also
-            int_dict = {i.interface_id: {'type': i.type, 'layer': i.overlay_id} for i in node.interfaces()}
-            int_dict = {i.interface_id: {'type': i.type} for i in node.interfaces()}
+            int_dict = {i.interface_id: {'type': i.type, 
+                'description': i.description,
+                'layer': i.overlay_id} for i in node.interfaces()}
+            int_dict = {i.interface_id: {'type': i.type,
+                'description': i.description,
+                } for i in node.interfaces()}
             self._graph.node[node.node_id]["_interfaces"] = int_dict
 
     def add_edge(self, src, dst, retain=[], **kwargs):
